@@ -124,7 +124,7 @@ void KDirListerTest::testOpenUrl()
 
     KFileItem rootByUrl = m_dirLister.findByUrl(path);
     QVERIFY(!rootByUrl.isNull());
-    QCOMPARE(QString(rootByUrl.url().path() + '/'), path);
+    QCOMPARE(QString(rootByUrl.url().toLocalFile() + '/'), path);
 
     m_dirLister.clearSpies(); // for the tests that call testOpenUrl for setup
 }
@@ -318,7 +318,7 @@ void KDirListerTest::testNewItemsInSymlink() // #213799
         }
         QCOMPARE(dirLister2.spyDeleteItem.count(), 1);
         const KFileItem item = dirLister2.spyDeleteItem[0][0].value<KFileItem>();
-        QCOMPARE(item.url().path(), QString(symPath + '/' + fileName));
+        QCOMPARE(item.url().toLocalFile(), QString(symPath + '/' + fileName));
     }
 
     // TODO: test file update.
@@ -356,10 +356,10 @@ void KDirListerTest::testRefreshItems()
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), fileName);
+    QCOMPARE(entry.first.url().toLocalFile(), fileName);
     QCOMPARE(entry.first.size(), KIO::filesize_t(11));
     QCOMPARE(entry.first.mimetype(), QString("application/octet-stream"));
-    QCOMPARE(entry.second.url().path(), fileName);
+    QCOMPARE(entry.second.url().toLocalFile(), fileName);
     QCOMPARE(entry.second.size(), KIO::filesize_t(11 /*Hello world*/ + 6 /*<html>*/));
     QCOMPARE(entry.second.mimetype(), QString("text/html"));
 
@@ -398,15 +398,15 @@ void KDirListerTest::testRefreshRootItem()
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
     QCOMPARE(entry.first.name(), QString("subdir"));
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
     QCOMPARE(entry.second.name(), QString("subdir"));
 
     QCOMPARE(m_refreshedItems2.count(), 1);
     entry = m_refreshedItems2.first();
-    QCOMPARE(entry.first.url().path(), path);
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
     // item name() doesn't matter here, it's the root item.
 
     m_refreshedItems.clear();
@@ -424,8 +424,8 @@ void KDirListerTest::testRefreshRootItem()
     waitForRefreshedItems();
     QCOMPARE(m_refreshedItems.count(), 1);
     entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), path);
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
 
     m_refreshedItems.clear();
     m_refreshedItems2.clear();
@@ -488,16 +488,16 @@ void KDirListerTest::testRenameItem()
 
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), path);
     QCOMPARE(entry.first.mimetype(), QString("application/octet-stream"));
-    QCOMPARE(entry.second.url().path(), newPath);
+    QCOMPARE(entry.second.url().toLocalFile(), newPath);
     QCOMPARE(entry.second.mimetype(), QString("text/html"));
     disconnect(&m_dirLister, 0, this, 0);
 
     // Let's see what KDirLister has in cache now
     KFileItem cachedItem = m_dirLister.findByUrl(KUrl(newPath));
     QVERIFY(!cachedItem.isNull());
-    QCOMPARE(cachedItem.url().path(), newPath);
+    QCOMPARE(cachedItem.url().toLocalFile(), newPath);
     KFileItem oldCachedItem = m_dirLister.findByUrl(KUrl(path));
     QVERIFY(oldCachedItem.isNull());
     m_refreshedItems.clear();
@@ -517,7 +517,7 @@ void KDirListerTest::testRenameAndOverwrite() // has to be run after testRenameI
         QTest::qWait(100);
         existingItem = m_dirLister.findByUrl(KUrl(path));
     };
-    QCOMPARE(existingItem.url().path(), path);
+    QCOMPARE(existingItem.url().toLocalFile(), path);
 
     m_refreshedItems.clear();
     connect(&m_dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)),
@@ -538,13 +538,13 @@ void KDirListerTest::testRenameAndOverwrite() // has to be run after testRenameI
 
     QCOMPARE(m_refreshedItems.count(), 1);
     QPair<KFileItem, KFileItem> entry = m_refreshedItems.first();
-    QCOMPARE(entry.first.url().path(), newPath);
-    QCOMPARE(entry.second.url().path(), path);
+    QCOMPARE(entry.first.url().toLocalFile(), newPath);
+    QCOMPARE(entry.second.url().toLocalFile(), path);
     disconnect(&m_dirLister, 0, this, 0);
 
     // Let's see what KDirLister has in cache now
     KFileItem cachedItem = m_dirLister.findByUrl(KUrl(path));
-    QCOMPARE(cachedItem.url().path(), path);
+    QCOMPARE(cachedItem.url().toLocalFile(), path);
     KFileItem oldCachedItem = m_dirLister.findByUrl(KUrl(newPath));
     QVERIFY(oldCachedItem.isNull());
     m_refreshedItems.clear();
@@ -871,6 +871,95 @@ void KDirListerTest::testOpenAndStop()
     disconnect(&m_dirLister, 0, this, 0);
 }
 
+// A bug in the decAutoUpdate/incAutoUpdate logic made KDirLister stop watching a directory for changes,
+// and never watch it again when opening it from the cache.
+void KDirListerTest::testBug211472()
+{
+    m_items.clear();
+
+    KTempDir newDir;
+    const QString path = newDir.name() + "newsubdir/";
+    QDir().mkdir(path);
+    MyDirLister dirLister;
+    connect(&dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
+
+    dirLister.openUrl(KUrl(path));
+    QVERIFY(QTest::kWaitForSignal(&dirLister, SIGNAL(completed()), 1000));
+    QVERIFY(dirLister.isFinished());
+    QVERIFY(m_items.isEmpty());
+
+    if (true) {
+        // This block is required to trigger bug 211472.
+
+        // Go 'up' to the parent of 'newsubdir'.
+        dirLister.openUrl(KUrl(newDir.name()));
+        QVERIFY(QTest::kWaitForSignal(&dirLister, SIGNAL(completed()), 1000));
+        QVERIFY(dirLister.isFinished());
+        QVERIFY(!m_items.isEmpty());
+        m_items.clear();
+
+        // Create a file in 'newsubdir' while we are listing its parent dir.
+        createTestFile(path + "newFile-1");
+        // At this point, newsubdir is not used, so it's moved to the cache.
+        // This happens in checkUpdate, called when receiving a notification for the cached dir,
+        // this is why this unittest needs to create a test file in the subdir.
+        QTest::qWait(1000);
+        QVERIFY(m_items.isEmpty());
+
+        // Return to 'newsubdir'. It will be emitted from the cache, then an update will happen.
+        dirLister.openUrl(KUrl(path));
+        QVERIFY(QTest::kWaitForSignal(&dirLister, SIGNAL(completed()), 1000));
+        QVERIFY(QTest::kWaitForSignal(&dirLister, SIGNAL(completed()), 1000));
+        QVERIFY(dirLister.isFinished());
+        QCOMPARE(m_items.count(), 1);
+        m_items.clear();
+    }
+
+    // Now try to create a second file in 'newsubdir' and verify that the
+    // dir lister notices it.
+    QTest::qWait(1000); // We need a 1s timestamp difference on the dir, otherwise FAM won't notice anything.
+
+    createTestFile(path + "newFile-2");
+
+    int numTries = 0;
+    // Give time for KDirWatch to notify us
+    while (m_items.isEmpty()) {
+        QVERIFY(++numTries < 10);
+        QTest::qWait(200);
+    }
+    QCOMPARE(m_items.count(), 1);
+
+    newDir.unlink();
+    QVERIFY(QTest::kWaitForSignal(&dirLister, SIGNAL(clear()), 1000));
+}
+
+void KDirListerTest::testRenameCurrentDir() // #294445
+{
+    m_items.clear();
+
+    const QString path = m_tempDir.name() + "newsubdir-1";
+    QVERIFY(QDir().mkdir(path));
+    MyDirLister secondDirLister;
+    connect(&secondDirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
+
+    secondDirLister.openUrl(KUrl(path));
+    QVERIFY(QTest::kWaitForSignal(&secondDirLister, SIGNAL(completed()), 1000));
+    QVERIFY(secondDirLister.isFinished());
+    QVERIFY(m_items.empty());
+    QCOMPARE(secondDirLister.rootItem().url().toLocalFile(), path);
+
+    const QString newPath = m_tempDir.name() + "newsubdir-2";
+    QVERIFY(QDir().rename(path, newPath));
+    org::kde::KDirNotify::emitFileRenamed(KUrl(path).url(), KUrl(newPath).url());
+    QVERIFY(QTest::kWaitForSignal(&secondDirLister, SIGNAL(redirection(KUrl,KUrl)), 1000));
+
+    // Check that the URL of the root item got updated
+    QCOMPARE(secondDirLister.rootItem().url().toLocalFile(), newPath);
+
+    disconnect(&secondDirLister, 0, this, 0);
+    QDir().remove(newPath);
+}
+
 void KDirListerTest::testRedirection()
 {
     m_items.clear();
@@ -911,6 +1000,62 @@ void KDirListerTest::testRedirection()
 
 }
 
+void KDirListerTest::testWatchingAfterCopyJob() // #331582
+{
+    m_items.clear();
+
+    KTempDir newDir;
+    const QString path = newDir.name();
+
+    // List and watch an empty dir
+    connect(&m_dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
+    m_dirLister.openUrl(KUrl(path));
+    QVERIFY(QTest::kWaitForSignal(&m_dirLister, SIGNAL(completed()), 1000));
+    QVERIFY(m_dirLister.isFinished());
+    QVERIFY(m_items.isEmpty());
+
+    // Create three subfolders.
+    QVERIFY(QDir().mkdir(path + "New Folder"));
+    QVERIFY(QDir().mkdir(path + "New Folder 1"));
+    QVERIFY(QDir().mkdir(path + "New Folder 2"));
+
+    QVERIFY(QTest::kWaitForSignal(&m_dirLister, SIGNAL(completed()), 1000));
+    QVERIFY(m_dirLister.isFinished());
+    QCOMPARE(m_items.count(), 3);
+
+    // Create a new file and verify that the dir lister notices it.
+    m_items.clear();
+    createTestFile(path + "a");
+    QVERIFY(QTest::kWaitForSignal(&m_dirLister, SIGNAL(completed()), 1000));
+    QVERIFY(m_dirLister.isFinished());
+    QCOMPARE(m_items.count(), 1);
+
+    // Rename one of the subfolders.
+    const QString oldPath = path + "New Folder 1";
+    const QString newPath = path + "New Folder 1a";
+
+    // NOTE: The following two lines are required to trigger the bug!
+    KIO::Job* job = KIO::moveAs(KUrl(oldPath), KUrl(newPath), KIO::HideProgressInfo);
+    job->exec();
+
+    // Now try to create a second new file in and verify that the
+    // dir lister notices it.
+    m_items.clear();
+    createTestFile(path + "b");
+
+    int numTries = 0;
+    // Give time for KDirWatch to notify us
+    // This should end up in "KDirListerCache::slotFileDirty"
+    while (m_items.isEmpty()) {
+        QVERIFY(++numTries < 10);
+        QTest::qWait(200);
+    }
+    QCOMPARE(m_items.count(), 1);
+
+    newDir.unlink();
+    QVERIFY(QTest::kWaitForSignal(&m_dirLister, SIGNAL(clear()), 1000));
+}
+
 void KDirListerTest::enterLoop(int exitCount)
 {
     //qDebug("enterLoop");
@@ -964,8 +1109,14 @@ void KDirListerTest::testDeleteCurrentDir()
     enterLoop();
     QCOMPARE(m_dirLister.spyClear.count(), 1);
     QCOMPARE(m_dirLister.spyClearKUrl.count(), 0);
-    QCOMPARE(m_dirLister.spyItemsDeleted.count(), 1);
-    QCOMPARE(m_dirLister.spyItemsDeleted[0][0].value<KFileItemList>().count(), 1);
+    KUrl::List deletedUrls;
+    for (int i = 0; i < m_dirLister.spyItemsDeleted.count(); ++i)
+        deletedUrls += m_dirLister.spyItemsDeleted[i][0].value<KFileItemList>().urlList();
+    //kDebug() << deletedUrls;
+    KUrl currentDirUrl = QUrl::fromLocalFile(path());
+    currentDirUrl.adjustPath(KUrl::RemoveTrailingSlash);
+    // Sometimes I get ("current/subdir", "current") here, but that seems ok.
+    QVERIFY(deletedUrls.contains(currentDirUrl));
 }
 
 int KDirListerTest::fileCount() const
